/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 *
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.mail.service.consume;

import java.util.List;
import java.util.TimerTask;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange.Role;
import javax.mail.Flags.Flag;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;

import org.ow2.petals.bc.mail.MailConsumeFlowStepBeginLogData;
import org.ow2.petals.bc.mail.MailSessionManager;
import org.ow2.petals.bc.mail.listeners.ExternalListener;
import org.ow2.petals.commons.log.Level;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.logger.Utils;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.thread.ExecutionContext;
import com.ebmwebsourcing.easycommons.uuid.SimpleUUIDGenerator;

/**
 * This class is used to monitor a mailbox for new mails (accept() method).
 * <p>
 * It acts as a Listener. After new mail retrieval, the listener calls a
 * 'process(MimeMessage)' method on the MimeMessageManager, which acts as a
 * handler of MimeMessages.
 * </p>
 *
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 */
public class ExternalEventPoller extends TimerTask {

    private static final SimpleUUIDGenerator simpleUUIDGenerator;

    static {
        simpleUUIDGenerator = new SimpleUUIDGenerator();
    }

    private final transient ExternalListener externalListener;

    protected final transient Logger logger;

    protected transient MailSessionManager mailSessionManager;

    protected transient Consumes consumes;

    protected transient ConsumeDescriptor consumeDescriptor;

    /**
     * Instantiate a {@link ExternalEventPoller} for a specific address
     *
     * @param externalListener
     *            the external listener for a given consumes
     * @param log
     *            the component logger
     * @param consumeDescriptor
     *            information about the mail session to open
     * @param mailSessionManager
     *            the {@link MailSessionManager}
     */
    public ExternalEventPoller(final Logger log, ExternalListener externalListener,
            Consumes consumes, ConsumeDescriptor consumeDescriptor,
            MailSessionManager mailSessionManager) {
        super();
        this.logger = log;
        this.externalListener = externalListener;
        this.mailSessionManager = mailSessionManager;
        this.consumes = consumes;
        this.consumeDescriptor = consumeDescriptor;
    }

    public Logger getLogger() {
        return this.logger;
    }

    /**
     * Create mail store and folder. Retrieve new mails and process them. Mail
     * store and folder are opened (connected) and closed each time the run
     * method is called. If the processing of a message fails, it is skipped and
     * the processing of other messages continue. All processed messages are
     * deleted from the folder.
     */
    @Override
    public void run() {
        List<Message> messages = null;
        Folder folder = null;
        Store store = null;
        try {
            // Create a Session
            final Session session = this.mailSessionManager
                    .createSessionPropertiesFromDescriptor(this.consumeDescriptor);

            // Get mail store and connect to it
            store = this.mailSessionManager.getStoreAndConnect(session, this.consumeDescriptor);

            // Get mail folder and open it
            folder = this.mailSessionManager.getFolderAndOpen(store, this.consumeDescriptor);

            // receive messages
            messages = this.mailSessionManager.getNewMails(folder);

            // Process messages. If the processing of a message fails, it is
            // skipped and the processing of other messages continue
            for (final Message message : messages) {
                try {
                    PetalsExecutionContext.putFlowInstanceId(simpleUUIDGenerator.getNewID());
                    PetalsExecutionContext.putFlowStepId(simpleUUIDGenerator.getNewID());

                    String flowInstanceId = ExecutionContext.getProperties().getProperty(
                            PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME);
                    String flowStepId = ExecutionContext.getProperties().getProperty(
                            PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME);
                    getLogger().log(
                            Level.MONIT,
                            "",
                            new MailConsumeFlowStepBeginLogData(flowInstanceId, flowStepId,
                                    StringHelper.nonNullValue(this.consumes.getInterfaceName()),
                                    StringHelper.nonNullValue(this.consumes.getServiceName()), this.consumes
                                            .getEndpointName(), StringHelper.nonNullValue(this.consumes
                                            .getOperation()), StringHelper
                                            .nonNullValue(this.consumeDescriptor.getScheme()),
                                    StringHelper.nonNullValue(this.consumeDescriptor.getHostname()),
                                    StringHelper.nonNullValue(this.consumeDescriptor.getPort()),
                                    StringHelper.nonNullValue(this.consumeDescriptor.getUsername()),
                                    StringHelper.nonNullValue(this.consumeDescriptor.getFolder())));

                    this.externalListener.process( session, message );
                    message.setFlag(Flag.DELETED, true);
                } catch (Throwable t) {
                    this.logger.log(Level.WARNING, "Error when sending JBI request", t);
                    Utils.addMonitFailureTrace(this.logger,
                            PetalsExecutionContext.getFlowAttributes(), t.getMessage(),
                            Role.CONSUMER);
                }
            }
        } catch (final MessagingException e) {
            this.getLogger().log(Level.WARNING, e.getMessage(), e);
            return;
        } finally {
            this.mailSessionManager.closeFolderAndStore(folder, store,
                    this.consumeDescriptor.isDelete());
        }
    }
}
